<?php
/**
 * @package MediaWiki
 * @subpackage Maintenance
 */

 /** */

require_once 'convertLinks.inc';
require_once 'InitialiseMessages.inc';
require_once 'userDupes.inc';

$wgRenamedTables = array(
#           from             to                  patch file
#	array( 'group',         'groups',           'patch-rename-group.sql' ),
);

$wgNewTables = array(
#            table          patch file (in maintenance/archives)
	array( 'hitcounter',    'patch-hitcounter.sql' ),
	array( 'querycache',    'patch-querycache.sql' ),
	array( 'objectcache',   'patch-objectcache.sql' ),
	array( 'categorylinks', 'patch-categorylinks.sql' ),
	array( 'logging',       'patch-logging.sql' ),
	array( 'validate',      'patch-validate.sql' ),
	array( 'user_newtalk',  'patch-usernewtalk2.sql' ),
	array( 'transcache',	'patch-transcache.sql' ),
	array( 'trackbacks',	'patch-trackbacks.sql' ),
	array( 'externallinks', 'patch-externallinks.sql' ),
	array( 'job',           'patch-job.sql' ),
);

$wgNewFields = array(
#           table            field             patch file (in maintenance/archives)
	array( 'ipblocks',      'ipb_id',           'patch-ipblocks.sql' ),
	array( 'ipblocks',      'ipb_expiry',       'patch-ipb_expiry.sql' ),
	array( 'recentchanges', 'rc_type',          'patch-rc_type.sql' ),
	array( 'recentchanges', 'rc_ip',            'patch-rc_ip.sql' ),
	array( 'recentchanges', 'rc_id',            'patch-rc_id.sql' ),
	array( 'recentchanges', 'rc_patrolled',     'patch-rc-patrol.sql' ),
	array( 'user',          'user_real_name',   'patch-user-realname.sql' ),
	array( 'user',          'user_token',       'patch-user_token.sql' ),
	array( 'user',          'user_email_token', 'patch-user_email_token.sql' ),
	array( 'user',          'user_registration','patch-user_registration.sql' ),
 	array( 'logging',       'log_params',       'patch-log_params.sql' ),
 	array( 'archive',       'ar_rev_id',        'patch-archive-rev_id.sql' ),
 	array( 'archive',       'ar_text_id',       'patch-archive-text_id.sql' ),
 	array( 'page',          'page_len',         'patch-page_len.sql' ),
 	array( 'revision',      'rev_deleted',      'patch-rev_deleted.sql' ),
	array( 'image',         'img_width',        'patch-img_width.sql' ),
	array( 'image',         'img_metadata',     'patch-img_metadata.sql' ),
	array( 'image',         'img_media_type',   'patch-img_media_type.sql' ),
	array( 'validate',      'val_ip',           'patch-val_ip.sql' ),
	array( 'site_stats',    'ss_total_pages',   'patch-ss_total_articles.sql' ),
	array( 'interwiki',     'iw_trans',         'patch-interwiki-trans.sql' ),
	array( 'ipblocks',      'ipb_range_start',  'patch-ipb_range_start.sql' ),
	array( 'site_stats',    'ss_images',        'patch-ss_images.sql' ),
);

function rename_table( $from, $to, $patch ) {
	global $wgDatabase;
	if ( $wgDatabase->tableExists( $from ) ) {
		if ( $wgDatabase->tableExists( $to ) ) {
			wfOut( "...can't move table $from to $to, $to already exists.\n" );
		} else {
			wfOut( "Moving table $from to $to..." );
			dbsource( archive($patch), $wgDatabase );
			wfOut( "ok\n" );
		}
	} else {
		// Source table does not exist
		// Renames are done before creations, so this is typical for a new installation
		// Ignore silently
	}
}

function add_table( $name, $patch ) {
	global $wgDatabase;
	if ( $wgDatabase->tableExists( $name ) ) {
		wfOut( "...$name table already exists.\n" );
	} else {
		wfOut( "Creating $name table..." );
		dbsource( archive($patch), $wgDatabase );
		wfOut( "ok\n" );
	}
}

function add_field( $table, $field, $patch ) {
	global $wgDatabase;
	if ( !$wgDatabase->tableExists( $table ) ) {
		wfOut( "...$table table does not exist, skipping new field patch\n" );
	} elseif ( $wgDatabase->fieldExists( $table, $field ) ) {
		wfOut( "...have $field field in $table table.\n" );
	} else {
		wfOut( "Adding $field field to table $table..." );
		dbsource( archive($patch) , $wgDatabase );
		wfOut( "ok\n" );
	}
}

function do_revision_updates() {
	global $wgSoftwareRevision;
	if ( $wgSoftwareRevision < 1001 ) {
		update_passwords();
	}
}

function update_passwords() {
	wfDebugDieBacktrace( "This function needs to be updated or removed.\n" );

	global $wgDatabase;
	$fname = "Update script: update_passwords()";
	wfOut( "\nIt appears that you need to update the user passwords in your\n" .
	  "database. If you have already done this (if you've run this update\n" .
	  "script once before, for example), doing so again will make all your\n" .
	  "user accounts inaccessible, so be sure you only do this once.\n" .
	  "Update user passwords? (yes/no)" );

	$resp = readconsole();
    if ( ! ( "Y" == $resp{0} || "y" == $resp{0} ) ) { return; }

	$sql = "SELECT user_id,user_password FROM user";
	$source = $wgDatabase->query( $sql, $fname );

	while ( $row = $wgDatabase->fetchObject( $source ) ) {
		$id = $row->user_id;
		$oldpass = $row->user_password;
		$newpass = md5( "{$id}-{$oldpass}" );

		$sql = "UPDATE user SET user_password='{$newpass}' " .
		  "WHERE user_id={$id}";
		$wgDatabase->query( $sql, $fname );
	}
}

function do_interwiki_update() {
	# Check that interwiki table exists; if it doesn't source it
	global $wgDatabase, $IP;
	if( $wgDatabase->tableExists( "interwiki" ) ) {
		wfOut( "...already have interwiki table\n" );
		return true;
	}
	wfOut( "Creating interwiki table: " );
	dbsource( archive("patch-interwiki.sql") );
	wfOut( "ok\n" );
	wfOut( "Adding default interwiki definitions: " );
	dbsource( "$IP/maintenance/interwiki.sql" );
	wfOut( "ok\n" );
}

function do_index_update() {
	# Check that proper indexes are in place
	global $wgDatabase;
	$meta = $wgDatabase->fieldInfo( "recentchanges", "rc_timestamp" );
	if( $meta->multiple_key == 0 ) {
		wfOut( "Updating indexes to 20031107: " );
		dbsource( archive("patch-indexes.sql") );
		wfOut( "ok\n" );
		return true;
	}
	wfOut( "...indexes seem up to 20031107 standards\n" );
	return false;
}

function do_image_index_update() {
	global $wgDatabase;

	$meta = $wgDatabase->fieldInfo( "image", "img_major_mime" );
	if( $meta->multiple_key == 0 ) {
		wfOut( "Updating indexes to 20050912: " );
		dbsource( archive("patch-mimesearch-indexes.sql") );
		wfOut( "ok\n" );
		return true;
	}
	wfOut( "...indexes seem up to 20050912 standards\n" );
	return false;
}

function do_image_name_unique_update() {
	global $wgDatabase;
	if( $wgDatabase->indexExists( 'image', 'PRIMARY' ) ) {
		wfOut( "...image primary key already set.\n" );
	} else {
		wfOut( "Making img_name the primary key... " );
		dbsource( archive("patch-image_name_primary.sql"), $wgDatabase );
		wfOut( "ok\n" );
	}
}

function do_logging_timestamp_index() {
	global $wgDatabase;
	if( $wgDatabase->indexExists( 'logging', 'times' ) ) {
		wfOut( "...timestamp key on logging already exists.\n" );
	} else {
		wfOut( "Adding timestamp key on logging table... " );
		dbsource( archive("patch-logging-times-index.sql"), $wgDatabase );
		wfOut( "ok\n" );
	}
}


function do_watchlist_update() {
	global $wgDatabase;
	$fname = 'do_watchlist_update';
	if( $wgDatabase->fieldExists( 'watchlist', 'wl_notificationtimestamp' ) ) {
		wfOut( "The watchlist table is already set up for email notification.\n" );
	} else {
		wfOut( "Adding wl_notificationtimestamp field for email notification management." );
		/* ALTER TABLE watchlist ADD (wl_notificationtimestamp varchar(14) binary NOT NULL default '0'); */
		dbsource( archive( 'patch-email-notification.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	}
	# Check if we need to add talk page rows to the watchlist
	$talk = $wgDatabase->selectField( 'watchlist', 'count(*)', 'wl_namespace & 1', $fname );
	$nontalk = $wgDatabase->selectField( 'watchlist', 'count(*)', 'NOT (wl_namespace & 1)', $fname );
	if ( $talk != $nontalk ) {
		wfOut( "Adding missing watchlist talk page rows... " );
		flush();

		$wgDatabase->insertSelect( 'watchlist', 'watchlist', 
			array(
				'wl_user' => 'wl_user',
				'wl_namespace' => 'wl_namespace | 1',
				'wl_title' => 'wl_title',
				'wl_notificationtimestamp' => 'wl_notificationtimestamp'
			), array( 'NOT (wl_namespace & 1)' ), $fname, 'IGNORE' );
		wfOut( "ok\n" );
	} else {
		wfOut( "...watchlist talk page rows already present\n" );
	}
}

function do_copy_newtalk_to_watchlist() {
	global $wgDatabase;
	global $wgCommandLineMode;	# this needs to be saved while getID() and getName() are called

	$res = $wgDatabase->safeQuery( 'SELECT user_id, user_ip FROM !',
		$wgDatabase->tableName( 'user_newtalk' ) );
	$num_newtalks=$wgDatabase->numRows($res);
	wfOut( "Now converting $num_newtalks user_newtalk entries to watchlist table entries ... \n" );

	$user = new User();
	for ( $i = 1; $i <= $num_newtalks; $i++ ) {
		$wluser = $wgDatabase->fetchObject( $res );
		if ($wluser->user_id == 0) { # anonymous users ... have IP numbers as "names"
			if ($user->isIP($wluser->user_ip)) { # do only if it really looks like an IP number (double checked)
				$wgDatabase->replace( 'watchlist',
					array(array('wl_user','wl_namespace', 'wl_title', 'wl_notificationtimestamp' )),
					  array('wl_user' 			=> 0,
						'wl_namespace' 			=> NS_USER_TALK,
						'wl_title' 			=> $wluser->user_ip,
						'wl_notificationtimestamp' 	=> '19700101000000'
						), 'updaters.inc::do_watchlist_update2'
					);
			}
		} else { # normal users ... have user_ids
			$user->setID($wluser->user_id);
			$wgDatabase->replace( 'watchlist',
				array(array('wl_user','wl_namespace', 'wl_title', 'wl_notificationtimestamp' )),
				  array('wl_user' 			=> $user->getID(),
					'wl_namespace' 			=> NS_USER_TALK,
					'wl_title' 			=> $user->getName(),
					'wl_notificationtimestamp' 	=> '19700101000000'
					), 'updaters.inc::do_watchlist_update3'
				);
		}
	}
	wfOut( "Done.\n" );
}


function do_user_update() {
	global $wgDatabase;
	if( $wgDatabase->fieldExists( 'user', 'user_emailauthenticationtimestamp' ) ) {
		wfOut( "User table contains old email authentication field. Dropping... " );
		dbsource( archive( 'patch-email-authentication.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	} else {
		wfOut( "...user table does not contain old email authentication field.\n" );
	}
}

/**
 * 1.4 betas were missing the 'binary' marker from logging.log_title,
 * which causes a collation mismatch error on joins in MySQL 4.1.
 */
function do_logging_encoding() {
	global $wgDatabase, $wgDBtype;
	if ($wgDBtype != 'mysql')
		return;
	$logging = $wgDatabase->tableName( 'logging' );
	$res = $wgDatabase->query( "SELECT log_title FROM $logging LIMIT 0" );
	$flags = explode( ' ', mysql_field_flags( $res, 0 ) );
	$wgDatabase->freeResult( $res );

	if( in_array( 'binary', $flags ) ) {
		wfOut( "Logging table has correct title encoding.\n" );
	} else {
		wfOut( "Fixing title encoding on logging table... " );
		dbsource( archive( 'patch-logging-title.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	}
}

function do_schema_restructuring() {
	global $wgDatabase;
	$fname="do_schema_restructuring";
	if ( $wgDatabase->tableExists( 'page' ) ) {
		wfOut( "...page table already exists.\n" );
	} else {
		wfOut( "...converting from cur/old to page/revision/text DB structure.\n" );
		wfOut( wfTimestamp() );
		wfOut( "......checking for duplicate entries.\n" );

		extract( $wgDatabase->tableNames( 'cur', 'old', 'page', 'revision', 'text' ) );

		$rows = $wgDatabase->query( "SELECT cur_title, cur_namespace, COUNT(cur_namespace) AS c
				FROM $cur GROUP BY cur_title, cur_namespace HAVING c>1", $fname );

		if ( $wgDatabase->numRows( $rows ) > 0 ) {
			wfOut( wfTimestamp() );
			wfOut( "......<b>Found duplicate entries</b>\n" );
			wfOut( sprintf( "<b>      %-60s %3s %5s</b>\n", 'Title', 'NS', 'Count' ) );
			while ( $row = $wgDatabase->fetchObject( $rows ) ) {
				if ( ! isset( $duplicate[$row->cur_namespace] ) ) {
					$duplicate[$row->cur_namespace] = array();
				}
				$duplicate[$row->cur_namespace][] = $row->cur_title;
				wfOut( sprintf( "      %-60s %3s %5s\n", $row->cur_title, $row->cur_namespace, $row->c ) );
			}
			$sql = "SELECT cur_title, cur_namespace, cur_id, cur_timestamp FROM $cur WHERE ";
			$firstCond = true;
			foreach ( $duplicate as $ns => $titles ) {
				if ( $firstCond ) {
					$firstCond = false;
				} else {
					$sql .= ' OR ';
				}
				$sql .= "( cur_namespace = {$ns} AND cur_title in (";
				$first = true;
				foreach ( $titles as $t ) {
					if ( $first ) {
						$sql .= $wgDatabase->addQuotes( $t );
						$first = false;
					} else {
						$sql .= ', ' . $wgDatabase->addQuotes( $t );
					}
				}
				$sql .= ") ) \n";
			}
			# By sorting descending, the most recent entry will be the first in the list.
			# All following entries will be deleted by the next while-loop.
			$sql .= 'ORDER BY cur_namespace, cur_title, cur_timestamp DESC';

			$rows = $wgDatabase->query( $sql, $fname );

			$prev_title = $prev_namespace = false;
			$deleteId = array();

			while ( $row = $wgDatabase->fetchObject( $rows ) ) {
				if ( $prev_title == $row->cur_title && $prev_namespace == $row->cur_namespace ) {
					$deleteId[] = $row->cur_id;
				}
				$prev_title     = $row->cur_title;
				$prev_namespace = $row->cur_namespace;
			}
			$sql = "DELETE FROM $cur WHERE cur_id IN ( " . join( ',', $deleteId ) . ')';
			$rows = $wgDatabase->query( $sql, $fname );
			wfOut( wfTimestamp() );
			wfOut( "......<b>Deleted</b> ".$wgDatabase->affectedRows()." records.\n" );
		}


		wfOut( wfTimestamp() );
		wfOut( "......Creating tables.\n" );
		$wgDatabase->query("CREATE TABLE $page (
  			page_id int(8) unsigned NOT NULL auto_increment,
  			page_namespace int NOT NULL,
  			page_title varchar(255) binary NOT NULL,
  			page_restrictions tinyblob NOT NULL default '',
  			page_counter bigint(20) unsigned NOT NULL default '0',
  			page_is_redirect tinyint(1) unsigned NOT NULL default '0',
  			page_is_new tinyint(1) unsigned NOT NULL default '0',
  			page_random real unsigned NOT NULL,
  			page_touched char(14) binary NOT NULL default '',
  			page_latest int(8) unsigned NOT NULL,
  			page_len int(8) unsigned NOT NULL,

  			PRIMARY KEY page_id (page_id),
  			UNIQUE INDEX name_title (page_namespace,page_title),
  			INDEX (page_random),
  			INDEX (page_len)
			) TYPE=InnoDB", $fname );
		$wgDatabase->query("CREATE TABLE $revision (
  			rev_id int(8) unsigned NOT NULL auto_increment,
  			rev_page int(8) unsigned NOT NULL,
  			rev_comment tinyblob NOT NULL default '',
  			rev_user int(5) unsigned NOT NULL default '0',
  			rev_user_text varchar(255) binary NOT NULL default '',
  			rev_timestamp char(14) binary NOT NULL default '',
  			rev_minor_edit tinyint(1) unsigned NOT NULL default '0',
			rev_deleted tinyint(1) unsigned NOT NULL default '0',

  			PRIMARY KEY rev_page_id (rev_page, rev_id),
  			UNIQUE INDEX rev_id (rev_id),
  			INDEX rev_timestamp (rev_timestamp),
  			INDEX page_timestamp (rev_page,rev_timestamp),
  			INDEX user_timestamp (rev_user,rev_timestamp),
  			INDEX usertext_timestamp (rev_user_text,rev_timestamp)
			) TYPE=InnoDB", $fname );

		wfOut( wfTimestamp() );
		wfOut( "......Locking tables.\n" );
		$wgDatabase->query( "LOCK TABLES $page WRITE, $revision WRITE, $old WRITE, $cur WRITE", $fname );

		$maxold = intval( $wgDatabase->selectField( 'old', 'max(old_id)', '', $fname ) );
		wfOut( wfTimestamp() );
		wfOut( "......maxold is {$maxold}\n" );

		wfOut( wfTimestamp() );
		global $wgLegacySchemaConversion;
		if( $wgLegacySchemaConversion ) {
			// Create HistoryBlobCurStub entries.
			// Text will be pulled from the leftover 'cur' table at runtime.
			wfOut( "......Moving metadata from cur; using blob references to text in cur table.\n" );
			$cur_text = "concat('O:18:\"historyblobcurstub\":1:{s:6:\"mCurId\";i:',cur_id,';}')";
			$cur_flags = "'object'";
		} else {
			// Copy all cur text in immediately: this may take longer but avoids
			// having to keep an extra table around.
			wfOut( "......Moving text from cur.\n" );
			$cur_text = 'cur_text';
			$cur_flags = "''";
		}
		$wgDatabase->query( "INSERT INTO $old (old_namespace, old_title, old_text, old_comment, old_user, old_user_text,
    				old_timestamp, old_minor_edit, old_flags)
  			SELECT cur_namespace, cur_title, $cur_text, cur_comment, cur_user, cur_user_text, cur_timestamp, cur_minor_edit, $cur_flags
  			FROM $cur", $fname );

		wfOut( wfTimestamp() );
		wfOut( "......Setting up revision table.\n" );
		$wgDatabase->query( "INSERT INTO $revision (rev_id, rev_page, rev_comment, rev_user, rev_user_text, rev_timestamp,
    				rev_minor_edit)
			SELECT old_id, cur_id, old_comment, old_user, old_user_text,
    				old_timestamp, old_minor_edit
			FROM $old,$cur WHERE old_namespace=cur_namespace AND old_title=cur_title", $fname );

		wfOut( wfTimestamp() );
		wfOut( "......Setting up page table.\n" );
		$wgDatabase->query( "INSERT INTO $page (page_id, page_namespace, page_title, page_restrictions, page_counter,
    				page_is_redirect, page_is_new, page_random, page_touched, page_latest, page_len)
  			SELECT cur_id, cur_namespace, cur_title, cur_restrictions, cur_counter, cur_is_redirect, cur_is_new,
    				cur_random, cur_touched, rev_id, LENGTH(cur_text)
  			FROM $cur,$revision
  			WHERE cur_id=rev_page AND rev_timestamp=cur_timestamp AND rev_id > {$maxold}", $fname );

		wfOut( wfTimestamp() );
		wfOut( "......Unlocking tables.\n" );
		$wgDatabase->query( "UNLOCK TABLES", $fname );

		wfOut( wfTimestamp() );
		wfOut( "......Renaming old.\n" );
		$wgDatabase->query( "ALTER TABLE $old RENAME TO $text", $fname );

		wfOut( wfTimestamp() );
		wfOut( "...done.\n" );
	}
}

function do_inverse_timestamp() {
	global $wgDatabase;
	$fname="do_schema_restructuring";
	if( $wgDatabase->fieldExists( 'revision', 'inverse_timestamp' ) ) {
		wfOut( "Removing revision.inverse_timestamp and fixing indexes... " );
		dbsource( archive( 'patch-inverse_timestamp.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	} else {
		wfOut( "revision timestamp indexes already up to 2005-03-13\n" );
	}
}

function do_text_id() {
	global $wgDatabase;
	if( $wgDatabase->fieldExists( 'revision', 'rev_text_id' ) ) {
		wfOut( "...rev_text_id already in place.\n" );
	} else {
		wfOut( "Adding rev_text_id field... " );
		dbsource( archive( 'patch-rev_text_id.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	}
}

function do_namespace_size() {
	$tables = array(
		'page'          => 'page',
		'archive'       => 'ar',
		'recentchanges' => 'rc',
		'watchlist'     => 'wl',
		'querycache'    => 'qc',
		'logging'       => 'log',
	);
	foreach( $tables as $table => $prefix ) {
		do_namespace_size_on( $table, $prefix );
		flush();
	}
}

function do_namespace_size_on( $table, $prefix ) {
	global $wgDatabase, $wgDBtype;
	if ($wgDBtype != 'mysql')
		return;
	$field = $prefix . '_namespace';

	$tablename = $wgDatabase->tableName( $table );
	$result = $wgDatabase->query( "SHOW COLUMNS FROM $tablename LIKE '$field'" );
	$info = $wgDatabase->fetchObject( $result );
	$wgDatabase->freeResult( $result );

	if( substr( $info->Type, 0, 3 ) == 'int' ) {
		wfOut( "...$field is already a full int ($info->Type).\n" );
	} else {
		wfOut( "Promoting $field from $info->Type to int... " );

		$sql = "ALTER TABLE $tablename MODIFY $field int NOT NULL";
		$wgDatabase->query( $sql );

		wfOut( "ok\n" );
	}
}

function do_pagelinks_update() {
	global $wgDatabase;
	if( $wgDatabase->tableExists( 'pagelinks' ) ) {
		wfOut( "...already have pagelinks table.\n" );
	} else {
		wfOut( "Converting links and brokenlinks tables to pagelinks... " );
		dbsource( archive( 'patch-pagelinks.sql' ), $wgDatabase );
		wfOut( "ok\n" );
		flush();

		global $wgCanonicalNamespaceNames;
		foreach( $wgCanonicalNamespaceNames as $ns => $name ) {
			if( $ns != 0 ) {
				do_pagelinks_namespace( $ns );
			}
		}
	}
}

function do_pagelinks_namespace( $namespace ) {
	global $wgDatabase, $wgContLang;

	$ns = intval( $namespace );
	wfOut( "Cleaning up broken links for namespace $ns... " );

	$pagelinks = $wgDatabase->tableName( 'pagelinks' );
	$name = $wgContLang->getNsText( $ns );
	$prefix = $wgDatabase->strencode( $name );
	$likeprefix = str_replace( '_', '\\_', $prefix);

	$sql = "UPDATE $pagelinks
	           SET pl_namespace=$ns,
	               pl_title=TRIM(LEADING '$prefix:' FROM pl_title)
	         WHERE pl_namespace=0
	           AND pl_title LIKE '$likeprefix:%'";

	$wgDatabase->query( $sql, 'do_pagelinks_namespace' );
	wfOut( "ok\n" );
}

function do_drop_img_type() {
	global $wgDatabase;

	if( $wgDatabase->fieldExists( 'image', 'img_type' ) ) {
		wfOut( "Dropping unused img_type field in image table... " );
		dbsource( archive( 'patch-drop_img_type.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	} else {
		wfOut( "No img_type field in image table; Good.\n" );
	}
}

function do_old_links_update() {
	global $wgDatabase;
	if( $wgDatabase->tableExists( 'pagelinks' ) ) {
		wfOut( "Already have pagelinks; skipping old links table updates.\n" );
	} else {
		convertLinks(); flush();
	}
}

function do_user_unique_update() {
	global $wgDatabase;
	$duper = new UserDupes( $wgDatabase );
	if( $duper->hasUniqueIndex() ) {
		wfOut( "Already have unique user_name index.\n" );
	} else {
		if( !$duper->clearDupes() ) {
			wfOut( "WARNING: This next step will probably fail due to unfixed duplicates...\n" );
		}
		wfOut( "Adding unique index on user_name... " );
		dbsource( archive( 'patch-user_nameindex.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	}
}

function do_user_groups_update() {
	$fname = 'do_user_groups_update';
	global $wgDatabase;

	if( $wgDatabase->tableExists( 'user_groups' ) ) {
		wfOut( "...user_groups table already exists.\n" );
		return do_user_groups_reformat();
	}

	wfOut( "Adding user_groups table... " );
	dbsource( archive( 'patch-user_groups.sql' ), $wgDatabase );
	wfOut( "ok\n" );

	if( !$wgDatabase->tableExists( 'user_rights' ) ) {
		if( $wgDatabase->fieldExists( 'user', 'user_rights' ) ) {
			wfOut( "Upgrading from a 1.3 or older database? Breaking out user_rights for conversion..." );
			dbsource( archive( 'patch-user_rights.sql' ), $wgDatabase );
			wfOut( "ok\n" );
		} else {
			wfOut( "*** WARNING: couldn't locate user_rights table or field for upgrade.\n" );
			wfOut( "*** You may need to manually configure some sysops by manipulating\n" );
			wfOut( "*** the user_groups table.\n" );
			return;
		}
	}

	wfOut( "Converting user_rights table to user_groups... " );
	$result = $wgDatabase->select( 'user_rights',
		array( 'ur_user', 'ur_rights' ),
		array( "ur_rights != ''" ),
		$fname );

	while( $row = $wgDatabase->fetchObject( $result ) ) {
		$groups = array_unique(
			array_map( 'trim',
				explode( ',', $row->ur_rights ) ) );

		foreach( $groups as $group ) {
			$wgDatabase->insert( 'user_groups',
				array(
					'ug_user'  => $row->ur_user,
					'ug_group' => $group ),
				$fname );
		}
	}
	$wgDatabase->freeResult( $result );
	wfOut( "ok\n" );
}

function do_user_groups_reformat() {
	# Check for bogus formats from previous 1.5 alpha code.
	global $wgDatabase;
	$info = $wgDatabase->fieldInfo( 'user_groups', 'ug_group' );

	if( $info->type == 'int' ) {
		$oldug = $wgDatabase->tableName( 'user_groups' );
		$newug = $wgDatabase->tableName( 'user_groups_bogus' );
		wfOut( "user_groups is in bogus intermediate format. Renaming to $newug... " );
		$wgDatabase->query( "ALTER TABLE $oldug RENAME TO $newug" );
		wfOut( "ok\n" );

		wfOut( "Re-adding fresh user_groups table... " );
		dbsource( archive( 'patch-user_groups.sql' ), $wgDatabase );
		wfOut( "ok\n" );

		wfOut( "***\n" );
		wfOut( "*** WARNING: You will need to manually fix up user permissions in the user_groups\n" );
		wfOut( "*** table. Old 1.5 alpha versions did some pretty funky stuff...\n" );
		wfOut( "***\n" );
	} else {
		wfOut( "...user_groups is in current format.\n" );
	}

}

function do_watchlist_null() {
	# Make sure wl_notificationtimestamp can be NULL,
	# and update old broken items.
	global $wgDatabase;
	$info = $wgDatabase->fieldInfo( 'watchlist', 'wl_notificationtimestamp' );

	if( $info->not_null ) {
		wfOut( "Making wl_notificationtimestamp nullable... " );
		dbsource( archive( 'patch-watchlist-null.sql' ), $wgDatabase );
		wfOut( "ok\n" );
	} else {
		wfOut( "...wl_notificationtimestamp is already nullable.\n" );
	}

}

/**
 * @bug 3946
 */
function do_page_random_update() {
	global $wgDatabase;

	wfOut( "Setting page_random to a random value on rows where it equals 0..." );

	$page = $wgDatabase->tableName( 'page' );
	$wgDatabase->query( "UPDATE $page SET page_random = RAND() WHERE page_random = 0", 'do_page_random_update' );
	$rows = $wgDatabase->affectedRows();

	wfOut( "changed $rows rows\n" );
}

function do_templatelinks_update() {
	global $wgDatabase, $wgLoadBalancer;
	$fname = 'do_templatelinks_update';

	if ( $wgDatabase->tableExists( 'templatelinks' ) ) {
		wfOut( "...templatelinks table already exists\n" );
		return;
	}
	wfOut( "Creating templatelinks table...\n" );
	dbsource( archive('patch-templatelinks.sql'), $wgDatabase );
	wfOut( "Populating...\n" );
	if ( isset( $wgLoadBalancer ) && $wgLoadBalancer->getServerCount() > 1 ) {
		// Slow, replication-friendly update
		$res = $wgDatabase->select( 'pagelinks', array( 'pl_from', 'pl_namespace', 'pl_title' ),
			array( 'pl_namespace' => NS_TEMPLATE ), $fname );
		$count = 0;
		while ( $row = $wgDatabase->fetchObject( $res ) ) {
			$count = ($count + 1) % 100;
			if ( $count == 0 ) {
				if ( function_exists( 'wfWaitForSlaves' ) ) {
					wfWaitForSlaves( 10 );
				} else {
					sleep( 1 );
				}
			}
			$wgDatabase->insert( 'templatelinks',
				array(
					'tl_from' => $row->pl_from,
					'tl_namespace' => $row->pl_namespace,
					'tl_title' => $row->pl_title,
				), $fname
			);

		}
		$wgDatabase->freeResult( $res );
	} else {
		// Fast update
		$wgDatabase->insertSelect( 'templatelinks', 'pagelinks',
			array(
				'tl_from' => 'pl_from',
				'tl_namespace' => 'pl_namespace',
				'tl_title' => 'pl_title'
			), array(
				'pl_namespace' => 10
			), $fname
		);
	}
	wfOut( "Done. Please run maintenance/refreshLinks.php for a more thorough templatelinks update.\n" );
}

function do_all_updates( $doShared = false ) {
	global $wgNewTables, $wgNewFields, $wgRenamedTables, $wgSharedDB, $wgDatabase;

	$doUser = !$wgSharedDB || $doShared;

	# Rename tables
	foreach ( $wgRenamedTables as $tableRecord ) {
		rename_table( $tableRecord[0], $tableRecord[1], $tableRecord[2] );
	}

	# Add missing tables
	foreach ( $wgNewTables as $tableRecord ) {
		add_table( $tableRecord[0], $tableRecord[1] );
		flush();
	}

	# Add missing fields
	foreach ( $wgNewFields as $fieldRecord ) {
		if ( $fieldRecord[0] != 'user' || $doUser ) {
			add_field( $fieldRecord[0], $fieldRecord[1], $fieldRecord[2] );
		}
		flush();
	}

	# Do schema updates which require special handling
	do_interwiki_update(); flush();
	do_index_update(); flush();
	do_old_links_update(); flush();
	do_image_name_unique_update(); flush();
	do_watchlist_update(); flush();
	if ( $doUser ) {
		do_user_update(); flush();
	}
######	do_copy_newtalk_to_watchlist(); flush();
	do_logging_encoding(); flush();

	do_schema_restructuring(); flush();
	do_inverse_timestamp(); flush();
	do_text_id(); flush();
	do_namespace_size(); flush();

	do_pagelinks_update(); flush();
	do_templatelinks_update(); flush(); // after pagelinks

	do_drop_img_type(); flush();

	if ( $doUser ) {
		do_user_unique_update(); flush();
	}
	do_user_groups_update(); flush();

	do_watchlist_null(); flush();

	//do_image_index_update(); flush();

	do_logging_timestamp_index(); flush();

	do_page_random_update(); flush();

	initialiseMessages(); flush();
}

function archive($name) {
	global $wgDBtype, $IP;
	switch ($wgDBtype) {
	case "oracle":
		return "$IP/maintenance/oracle/archives/$name";
	default:
		return "$IP/maintenance/archives/$name";
	}
}
?>
